package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.domain.User;

import at.ac.uibk.gitsearch.security.jwt.JWTFilter;
import at.ac.uibk.gitsearch.security.jwt.TokenProvider;
import at.ac.uibk.gitsearch.service.UserService;
import at.ac.uibk.gitsearch.web.rest.vm.LoginVM;

import com.fasterxml.jackson.annotation.JsonProperty;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

import javax.validation.Valid;

/**
 * Controller to authenticate users.
 */
@RestController
@RequestMapping("/api")
public class UserJWTController {

    private final TokenProvider tokenProvider;
    
    private UserService userService;

    private final AuthenticationManagerBuilder authenticationManagerBuilder;

    public UserJWTController(TokenProvider tokenProvider,
    		AuthenticationManagerBuilder authenticationManagerBuilder,
    		UserService userService) {
        this.tokenProvider = tokenProvider;
        this.authenticationManagerBuilder = authenticationManagerBuilder;
        this.userService = userService;
    }

    @PostMapping("/authenticate")
    public ResponseEntity<JWTToken> authorize(@Valid @RequestBody LoginVM loginVM) {

        UsernamePasswordAuthenticationToken authenticationToken =
            new UsernamePasswordAuthenticationToken(loginVM.getUsername(), loginVM.getPassword());

        Authentication authentication = authenticationManagerBuilder.getObject().authenticate(authenticationToken);
        updateLastLogin(authentication);
        SecurityContextHolder.getContext().setAuthentication(authentication);
        boolean rememberMe = (loginVM.isRememberMe() == null) ? false : loginVM.isRememberMe();
        String jwt = tokenProvider.createToken(authentication, rememberMe);
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.add(JWTFilter.AUTHORIZATION_HEADER, "Bearer " + jwt);
        return new ResponseEntity<>(new JWTToken(jwt), httpHeaders, HttpStatus.OK);
    }

	private void updateLastLogin(Authentication authentication) {
		if (authentication.getPrincipal() instanceof User) {
			User user = (User) authentication.getPrincipal();
			userService.updateLastLogin(user.getId());
		} else if (authentication.getPrincipal() instanceof org.springframework.security.core.userdetails.User) {
			org.springframework.security.core.userdetails.User user = (org.springframework.security.core.userdetails.User) authentication.getPrincipal();
			userService.updateLastLogin(user.getUsername());
		}
	}
    
    @PostMapping("/refreshToken")
    public ResponseEntity<JWTToken> refreshToken(@RequestParam("token") String token) {
    	if(!tokenProvider.validateToken(token)) {
    		return new ResponseEntity<>(null, HttpStatus.UNAUTHORIZED);
    	} else {
    		 Authentication authentication = tokenProvider.getAuthentication(token);
    		 @SuppressWarnings("unchecked")
			Map<String, String> details = (Map<String, String>) authentication.getDetails();
    		 if(!details.containsKey(TokenProvider.PRE_TOKEN_CLAIM)) {
    	    		return new ResponseEntity<>(null, HttpStatus.UNAUTHORIZED);
    		 }
  	         updateLastLogin(authentication);
   	         SecurityContextHolder.getContext().setAuthentication(authentication);
   	        String jwt = tokenProvider.createToken(authentication, false);
   	        HttpHeaders httpHeaders = new HttpHeaders();
   	        httpHeaders.add(JWTFilter.AUTHORIZATION_HEADER, "Bearer " + jwt);
   	        return new ResponseEntity<>(new JWTToken(jwt), httpHeaders, HttpStatus.OK);
    	}
    }
    
    /**
     * Object to return as body in JWT Authentication.
     */
    static class JWTToken {

        private String idToken;

        JWTToken(String idToken) {
            this.idToken = idToken;
        }

        @JsonProperty("id_token")
        String getIdToken() {
            return idToken;
        }

        void setIdToken(String idToken) {
            this.idToken = idToken;
        }
    }
}
